<?php
/*
 * @Author: ofearn
 * @Date: 2019/10/16 16:24
 * @Last Modified by: ofearn@qq.com
 */

namespace app\controller;


use app\Db;
use app\lib\Random;
use app\service\Pns;
use app\service\Sms;
use app\service\User;
use app\service\login\QQ;
use app\service\login\Sina;
use app\service\login\Wechat;
use AppleSignIn\ASDecoder;

class Login extends Base
{
    protected $type;

    public function __construct($controller = '')
    {
        parent::__construct($controller);
    }

    public function get_oauth_url()
    {
        //Need oauth jump pages
        $backUrl = [
            'qq' => $this->siteUrl . 'plugin.php?id=yimen_app&s=login/qq',
            'pc_wechat' => $this->siteUrl . 'plugin.php?id=yimen_app&s=login/pc_wechat',
            'wechat' => $this->siteUrl . 'plugin.php?id=yimen_app&s=login/wechat',
            'sina' => $this->siteUrl . 'plugin.php?id=yimen_app&s=login/sina',
            'wechat_pay' => $this->siteUrl . 'plugin.php?id=yimen_app&s=pay/wechat_mp&order_id=' . param('order_id')
        ];
        $urlArr = [
            'qq' => 'https://graph.qq.com/oauth2.0/authorize?response_type=code&client_id=' . $this->set->get('login_qq.pc_appid')
                . '&redirect_uri=' . urlencode($backUrl['qq']) . '&state=yimen_app&scope=get_user_info,list_album,upload_pic,do_like',
            'pc_wechat' => 'https://open.weixin.qq.com/connect/qrconnect?appid=' . $this->set->get('login_wechat.pc_appid')
                . '&redirect_uri=' . urlencode($backUrl['pc_wechat']) . '&response_type=code&scope=snsapi_login&state=yimen#wechat_redirect',
            'wechat' => 'https://open.weixin.qq.com/connect/oauth2/authorize?appid=' . $this->set->get('login_wechat.wx_appid')
                . '&redirect_uri=' . urlencode($backUrl['wechat']) . '&response_type=code&scope=snsapi_base,snsapi_userinfo&state=yimemapp#wechat_redirect',
            'sina' => 'https://api.weibo.com/oauth2/authorize?client_id=' . $this->set->get('login_sina.pc_appkey')
                . '&response_type=code&redirect_uri=' . urlencode($backUrl['sina']),
            'wechat_pay' => 'https://open.weixin.qq.com/connect/oauth2/authorize?appid=' . $this->set->get('login_wechat.wx_appid')
                . '&redirect_uri=' . urlencode($backUrl['wechat_pay']) . '&response_type=code&scope=snsapi_userinfo&state=yimemapp#wechat_redirect',
        ];
        $type = param('type');
        if (empty($urlArr[$type])) {
            return json([
                'code' => 0,
                'msg' => '5b2T5YmN55m75b2V5pa55byP5LiN5a2Y5Zyo'
            ]);
        }
        return json([
            'code' => 1,
            'msg' => '6I635Y+W5oiQ5Yqf',
            'url' => $urlArr[$type]
        ]);
    }

    public function app_qq()
    {
        if ($this->set->get('login_qq.app_login') != 1) {
            return json([
                'code' => 0,
                'msg' => '55m75b2V5pa55byP5pyq5byA5ZCv'
            ]);
        }
        $request = $_REQUEST;
        $openid = $request['openid'];
        if (empty($openid)) {
            return json([
                'code' => 0,
                'msg' => 'UVHmjojmnYPkv6Hmga/plJnor68='
            ]);
        }
        $qq = new QQ('', '');
        $checkOpenid = $qq->setAccessToken($request['at'])->getOpenid();
        if ($checkOpenid != $openid) {
            return json([
                'code' => 0,
                'msg' => 'UVHmjojmnYPkv6Hmga/plJnor68='
            ]);
        }
        $this->userInfo = [
            'openid' => $openid,
            'unionid' => $qq->getUnionid() ?: '',
            'nick' => $request['nickname'],
            'avatar' => urldecode($request['figureurl'])
        ];
        $this->type = 'qq';
        return $this->toLogin();
    }

    public function app_wechat()
    {
        if ($this->set->get('login_wechat.app_login') != 1) {
            return json([
                'code' => 0,
                'msg' => '55m75b2V5pa55byP5pyq5byA5ZCv'
            ]);
        }
        $request = $_REQUEST;
        $code = $request['code'];
        if (empty($request['openid']) || empty($request['unionid'])) {
            $wechat = new Wechat($this->set->get('login_wechat.app_appid'), $this->set->get('login_wechat.app_secret'));
            $userInfo = $wechat->setCode($code)->getUserInfo();
            if (!$userInfo) {
                return json([
                    'code' => 0,
                    'msg' => '5b6u5L+h5o6I5p2D5L+h5oGv6ZSZ6K+v'
                ]);
            }
            $this->userInfo = $userInfo;
        } else {
            $this->userInfo = [
                'openid' => $request['openid'],
                'unionid' => $request['unionid'],
                'nick' => $request['nickname'],
                'avatar' => urldecode($request['headimgurl'])
            ];
        }
        $this->type = 'wechat';
        return $this->toLogin();
    }

    public function app_sina()
    {
        if ($this->set->get('login_sina.app_login') != 1) {
            return json([
                'code' => 0,
                'msg' => '55m75b2V5pa55byP5pyq5byA5ZCv'
            ]);
        }
        $request = $_REQUEST;
        $accessToken = $request['token'];
        $openid = $request['uid'];
        if (empty($accessToken) || empty($openid)) {
            return json([
                'code' => 0,
                'msg' => '5paw5rWq5o6I5p2D5L+h5oGv6ZSZ6K+v'
            ]);
        }
        $sina = new Sina($this->set->get('login_sina.app_appkey'), $this->set->get('login_sina.app_secret'));
        $userInfo = $sina->init($openid, $accessToken)->getUserInfo();
        $this->userInfo = $userInfo;
        $this->type = 'sina';
        return $this->toLogin();
    }

    private static function checkmobile()
    {
        global $_G;//全局变量，在discuz中使用，其他地方可忽略
        $mobile = array();
        static $mobilebrowser_list = array('iphone', 'android', 'phone', 'mobile', 'wap', 'netfront', 'java', 'opera mobi', 'opera mini',
            'ucweb', 'windows ce', 'symbian', 'series', 'webos', 'sony', 'blackberry', 'dopod', 'nokia', 'samsung',
            'palmsource', 'xda', 'pieplus', 'meizu', 'midp', 'cldc', 'motorola', 'foma', 'docomo', 'up.browser',
            'up.link', 'blazer', 'helio', 'hosin', 'huawei', 'novarra', 'coolpad', 'webos', 'techfaith', 'palmsource',
            'alcatel', 'amoi', 'ktouch', 'nexian', 'ericsson', 'philips', 'sagem', 'wellcom', 'bunjalloo', 'maui', 'smartphone',
            'iemobile', 'spice', 'bird', 'zte-', 'longcos', 'pantech', 'gionee', 'portalmmm', 'jig browser', 'hiptop',
            'benq', 'haier', '^lct', '320x320', '240x320', '176x220');
        $pad_list = array('pad', 'gt-p1000');

        $useragent = strtolower($_SERVER['HTTP_USER_AGENT']);

        if (dstrpos($useragent, $pad_list)) {
            return false;//当前为使用PAD用户，返回false
        }
        if (($v = dstrpos($useragent, $mobilebrowser_list, true))) {
            $_G['mobile'] = $v;
            return true;//手机端访问，返回true
        }
        $brower = array('mozilla', 'chrome', 'safari', 'opera', 'm3gate', 'winwap', 'openwave', 'myop');
        if (dstrpos($useragent, $brower)) return false;//PC浏览器访问，返回false

        $_G['mobile'] = 'unknown';//未能识别，根据访问的URL强制默认是否手机端
        if ($_GET['mobile'] === 'yes') {
            return true;
        } else {
            return false;
        }
    }

    public function qq()
    {
        if (self::checkmobile() && $this->set->get('login_qq.h5_login') != 1) {
            return $this->jump(false, ['msg' => '55m75b2V5pa55byP5pyq5byA5ZCv']);
        }
        if (!self::checkmobile() && $this->set->get('login_qq.pc_login') != 1) {
            return $this->jump(false, ['msg' => '55m75b2V5pa55byP5pyq5byA5ZCv']);
        }
        $request = $_REQUEST;
        $code = $request['code'];
        if (empty($code)) {
            return $this->jump(false, ['msg' => 'UVHmjojmnYPlpLHotKXvvIzor7fph43or5U=']);
        }
        $url = $this->siteUrl . 'plugin.php?id=yimen_app&s=login/qq';
        $qq = new QQ($this->set->get('login_qq.pc_appid'), $this->set->get('login_qq.pc_secret'));
        $userInfo = $qq->setCode($code)->setCallback($url)->getUserInfo();
        if (!$userInfo) {
            return $this->jump(false, ['msg' => 'UVHmjojmnYPkv6Hmga/plJnor68=']);
        }
        $this->userInfo = $userInfo;
        $this->type = 'qq';
        return $this->toLogin(false);
    }


    public function pc_wechat()
    {
        if ($this->set->get('login_wechat.pc_login') != 1) {
            return $this->jump(false, ['msg' => '55m75b2V5pa55byP5pyq5byA5ZCv']);
        }
        $request = $_REQUEST;
        $code = $request['code'];
        if (empty($code)) {
            return $this->jump(false, ['msg' => '5b6u5L+h5o6I5p2D5aSx6LSl77yM6K+36YeN6K+V']);
        }
        $wechat = new Wechat($this->set->get('login_wechat.pc_appid'), $this->set->get('login_wechat.pc_secret'));
        $userInfo = $wechat->setCode($code)->getUserInfo();
        if (!$userInfo) {
            return $this->jump(false, ['msg' => '5b6u5L+h5o6I5p2D5L+h5oGv6ZSZ6K+v']);
        }
        $this->userInfo = $userInfo;
        $this->type = 'wechat';
        return $this->toLogin(false);
    }

    public function sina()
    {
        if (self::checkmobile() && $this->set->get('login_sina.h5_login') != 1) {
            return $this->jump(false, ['msg' => '55m75b2V5pa55byP5pyq5byA5ZCv']);
        }
        if (!self::checkmobile() && $this->set->get('login_sina.pc_login') != 1) {
            return $this->jump(false, ['msg' => '55m75b2V5pa55byP5pyq5byA5ZCv']);
        }
        $request = $_REQUEST;
        $code = $request['code'];
        if (empty($code)) {
            return $this->jump(false, ['msg' => '5paw5rWq5o6I5p2D5aSx6LSl77yM6K+36YeN6K+V']);
        }
        $sina = new Sina($this->set->get('login_sina.pc_appkey'), $this->set->get('login_sina.pc_secret'));
        $url = $this->siteUrl . 'plugin.php?id=yimen_app&s=login/sina';
        $userInfo = $sina->setCode($code)->setCallback($url)->getUserInfo();
        if (!$userInfo) {
            return $this->jump(false, ['msg' => '5paw5rWq5o6I5p2D5L+h5oGv6ZSZ6K+v']);
        }
        $this->userInfo = $userInfo;
        $this->type = 'sina';
        return $this->toLogin(false);
    }

    public function wechat()
    {
        if ($this->set->get('login_wechat.wx_login') != 1) {
            return $this->jump(false, ['msg' => '55m75b2V5pa55byP5pyq5byA5ZCv']);
        }
        $request = $_REQUEST;
        $code = $request['code'];
        if (empty($code)) {
            return $this->jump(false, ['msg' => '5b6u5L+h5o6I5p2D5aSx6LSl77yM6K+36YeN6K+V']);
        }
        $wechat = new Wechat($this->set->get('login_wechat.wx_appid'), $this->set->get('login_wechat.wx_secret'));
        $userInfo = $wechat->setCode($code)->getUserInfo();
        if (!$userInfo) {
            return $this->jump(false, ['msg' => '5b6u5L+h5o6I5p2D5L+h5oGv6ZSZ6K+v']);
        }
        $this->userInfo = $userInfo;
        $this->type = 'wechat';
        return $this->toLogin(false);
    }

    /**
     * 获取验证码
     */
    public function captcha()
    {
        $config = $this->set->get('login_sms');
        if ($config['status'] != 1) {
            return json([
                'code' => 0,
                'msg' => '55m75b2V5pa55byP5pyq5byA5ZCv'
            ]);
        }
        $mobile = param('mobile');
        if (empty($mobile)) {
            return $this->error('5omL5py65Y+35LiN6IO95Li656m6');
        }
        $ip = get_ip();
        $time = time();
        if (intval($config['rate_num']) > 0) {
            $rate_time = intval($config['rate_time']) * 60;
            $count = Db::table('yimen_sms_code')->where('create_time', '>', $time - $rate_time)->where('ip', $ip)->count();
            if ($count >= intval($config['rate_num'])) {
                return $this->error('5Y+R6YCB6aKR546H6L+H5b+r77yM6K+356iN5ZCO5YaN6K+V77yB');
            }
        }
        $sms = new Sms($config);
        $code = Random::numeric(6);
        $result = $sms->sendCaptcha($mobile, $code);
        if ($result !== true) {
            return $this->error($result);
        }
        Db::table('yimen_sms_code')->insert([
            'mobile' => $mobile,
            'code' => $code,
            'status' => 0,
            'ip' => $ip,
            'create_time' => $time,
            'update_time' => $time
        ]);
        return $this->success('5Y+R6YCB5oiQ5Yqf');
    }

    /**
     * 手机号码登录
     */
    public function mobile()
    {
        $config = $this->set->get('login_sms');
        if ($config['status'] != 1) {
            return json([
                'code' => 0,
                'msg' => '55m75b2V5pa55byP5pyq5byA5ZCv'
            ]);
        }
        $mobile = param('mobile');
        $code = param('code');
        if (empty($mobile) || empty($code)) {
            return $this->error('5omL5py65Y+35ZKM6aqM6K+B56CB5LiN6IO95Li656m6');
        }
        $log = Db::table('yimen_sms_code')->where('mobile', $mobile)->where('code', $code)->first();
        if (!$log || $log['status'] != 0) {
            return $this->error('55+t5L+h6aqM6K+B56CB5LiN5q2j56Gu');
        }
        $timeout = $config['timeout'] * 60;
        if ($log['create_time'] + $timeout < time()) {
            return $this->error('55+t5L+h6aqM6K+B56CB5bey6L+H5pyf77yM6K+36YeN5paw6I635Y+W77yB');
        }
        Db::table('yimen_sms_code')->where('id', $log['id'])->update([
            'status' => 1,
            'update_time' => time()
        ]);
        $this->userInfo = [
            'openid' => $mobile,
            'unionid' => $mobile,
            'nick' => $config['pre'] . substr($mobile, -4),
            'avatar' => ''
        ];
        $this->type = 'mobile';
        return $this->toLogin();
    }

    public function get_pns()
    {
        $isIos = param('is_ios');
        return $this->success('6I635Y+W5oiQ5Yqf', [
            'key' => $isIos == 1 ? $this->set->get('login_pns.ios_key') : $this->set->get('login_pns.android_key')
        ]);
    }

    /**
     * 阿里手机登录
     */
    public function pns()
    {
        $config = $this->set->get('login_pns');
        if ($config['status'] != '1') {
            return $this->error('5pqC5pyq5byA5ZCv5Y+356CB6K6k6K+B55m75b2V');
        }
        $accessToken = trim(param('token'));
        if (empty($accessToken)) {
            return $this->error('YWNjZXNzVG9rZW7kuI3og73kuLrnqbo=');
        }
        $pns = new Pns([
            "accessKeyId" => $config['id'],
            "accessKeySecret" => $config['secret'],
        ]);
        $mobile = $pns->getMobile($accessToken);
        if (!$mobile) {
            return $this->error('5Y+356CB6K6k6K+B55m75b2V5aSx6LSl');
        }
        $this->userInfo = [
            'openid' => $mobile,
            'unionid' => $mobile,
            'nick' => $this->set->get('login_sms.pre') . substr($mobile, -4),
            'avatar' => ''
        ];
        $this->type = 'mobile';
        return $this->toLogin('5Y+356CB6K6k6K+B55m75b2V5oiQ5Yqf');
    }

    /**
     * ios登录
     */
    public function ios()
    {
        $config = $this->set->get('login_ios');
        if ($config['status'] != 1) {
            return $this->error('6Iu55p6c55m75b2V5pqC5pyq5byA5ZCv');
        }
        $user = trim(param('user'));
        $token = trim(param('token'));

        if (empty($user) || empty($token)) {
            return $this->error('5Y+C5pWw5LiN6IO95Li656m6');
        }
        $appleSignInPayload = ASDecoder::getAppleSignInPayload($token);
        $check = $appleSignInPayload->verifyUser($user);
        if (!$check) {
            return $this->error('55m75b2V5L+h5oGv5bey6L+H5pyf77yM6K+356iN5ZCO5YaN6K+V77yB');
        }
        $this->userInfo = [
            'openid' => $user,
            'unionid' => $user,
            'nick' => $config['pre'] . Random::alpha(6),
            'avatar' => ''
        ];
        $this->type = 'ios';
        return $this->toLogin();
    }

    protected function toLogin($isJson = true)
    {
        $info = $this->userInfo;
        $nick = sChar($info['nick']);
        $info['nick'] = base64_encode($info['nick']);
        if ($info['unionid'] != '') {
            $user = User::getByUnionid($info['unionid'], $this->type);
        } else {
            $user = User::getByOpenid($info['openid'], $this->type);
        }
        $isSave = $this->set->get('login_global.save_img') == 1 ? true : false;
        if ($user) {
            if ($user['uid'] == '') {
                User::upById($user['id'], array('update_time' => time()));
                return $this->jump($isJson, [
                    'code' => 2,
                    'msg' => '562J5b6F57uR5a6a',
                    'data' => array('token' => $info['openid']),
                    'url' => './source/plugin/yimen_app/static/h5/#/bind/' . $info['openid']
                ]);
            }
            if ($user['state'] == 0) {
                return $this->jump($isJson, [
                    'code' => 0,
                    'msg' => '55So5oi35bey6KKr5bCB56aB'
                ]);
            }
            if ($user['unionid'] == '' && $info['unionid'] != '') {
                User::upById($user['id'], array('unionid' => $info['unionid']));
            }
            if ($info['avatar'] != '' && $user['avatar'] != $info['avatar'] && $isSave) {
                User::upById($user['id'], array('avatar' => $info['avatar']));
                User::savaUserAvatar($user['id'], $info['avatar']);
            }
            wp_uid_Login($user['uid']);
            User::upById($user['id'], array('update_time' => time()));
            return $this->jump($isJson, [
                'code' => 1,
                'msg' => '55m75b2V5oiQ5Yqf'
            ]);
        } else {
            if ($this->set->get('login_global.is_bind') == 1 && $this->type !== 'mobile' && is_mobile()) {
                $ins = User::add(array(
                    'username' => '',
                    'password' => '',
                    'openid' => $info['openid'] ?: '',
                    'unionid' => $info['unionid'] ?: '',
                    'nick' => $info['nick'],
                    'avatar' => $info['avatar'],
                    'email' => '',
                    'platform' => $this->type,
                    'state' => 2,
                    'create_time' => time(),
                    'update_time' => time()
                ));
                if ($ins) {
                    return $this->jump($isJson, [
                        'code' => 2,
                        'msg' => '562J5b6F57uR5a6a',
                        'data' => array('token' => $info['openid']),
                        'url' => './source/plugin/yimen_app/static/h5/#/bind/' . $info['openid']
                    ]);
                } else {
                    return $this->jump($isJson, [
                        'code' => 0,
                        'msg' => '6YKu566x5qC85byP6ZSZ6K+v'
                    ]);
                }
            } else {
                $username = strtolower(Random::alpha(6));
                $password = Random::alnum(8);
                $usernames = $nick == "" ? $username : $nick;
                $r = User::checkUsername($usernames);
                if ($r != 1) {
                    $usernames = Random::alpha(6);
                }
                $uid = User::addUser($usernames, $password, $username . '@qq.com');
                if ($uid > 0) {
                    $ins = User::add(array(
                        'uid' => $uid,
                        'username' => $username,
                        'password' => $password,
                        'openid' => $info['openid'] ?: '',
                        'unionid' => $info['unionid'] ?: '',
                        'nick' => $info['nick'],
                        'avatar' => $info['avatar'],
                        'email' => $username . '@qq.com',
                        'platform' => $this->type,
                        'state' => 1,
                        'create_time' => time(),
                        'update_time' => time()
                    ));
                    $money = $this->set->get('login_global.send_money');
                    if ($money > 0) {
                        User::sendMoney($uid, $money, $this->set->get('pay_global.money_type'), 'login');
                    }
                    if (!$ins) {
                        return $this->jump($isJson, [
                            'code' => 0,
                            'msg' => '55m75b2V57mB5b+Z77yM56iN5ZCO5YaN6K+V'
                        ]);
                    } else {
                        if ($info['avatar'] != '' && $isSave) {
                            //Save Avatar
                            User::savaUserAvatar($uid, $info['avatar']);
                        }
                        wp_uid_Login($uid);
                        return $this->jump($isJson, [
                            'code' => 1,
                            'msg' => '55m75b2V5oiQ5Yqf'
                        ]);
                    }
                } else {
                    return $this->jump($isJson, [
                        'code' => 0,
                        'msg' => '6YKu566x5qC85byP6ZSZ6K+v'
                    ]);
                }
            }
        }
        return $this->jump($isJson, [
            'code' => 0,
            'msg' => 'err'
        ]);
    }
}