<?php


namespace app\yimen\push;


use GuzzleHttp\Client;

class Base
{
    protected $name;
    public $access_token;
    public $client;
    protected $key;
    protected $secret;

    public function __construct($key,$secret)
    {
        $this->client = new Client([
            'base_uri' => 'https://gate.open.yimenyun.com/',
        ]);
        $this->key=$key;
        $this->secret=$secret;
        $this->getToken();
    }

    public function getToken()
    {
        $ret = $this->client->post('token', [
            'form_params' => [
                'grant_type' => 'client_credentials',
                'client_id' => $this->key,
                'client_secret' => $this->secret
            ]
        ]);
        $res = json_decode($ret->getBody()->getContents(), true);
        if (isset($res['access_token'])) {
            $cacheData = [
                'token' => $res['access_token'],
                'timeout' => time() + intval($res['expires_in'])
            ];
            $this->access_token = $res['access_token'];
        }
    }

    protected function sendGet($url, $param)
    {
        try {
            $request = $this->client->get($url, [
                'headers' => [
                    'Authorization' => 'Bearer ' . $this->access_token
                ],
                'query' => $param
            ]);
            return json_decode($request->getBody()->getContents(), true);
        } catch (\Exception $e) {
            return [
                'code' => -1,
                'message' => 'api return ' . $e->getCode()
            ];
        }
    }

    protected function sendPost($url, $param)
    {
        try {
            $request = $this->client->post($url, [
                'headers' => [
                    'Authorization' => 'Bearer ' . $this->access_token
                ],
                'form_params' => $param
            ]);
            return json_decode($request->getBody()->getContents(), true);
        } catch (\Exception $e) {
            return [
                'code' => -1,
                'message' => 'api return ' . $e->getCode()
            ];
        }
    }
}